/*
    GARAGE GATE
*/

#include <stdio.h>
#include <system_def.h>
#include <cpu_def.h>
#include <ul_lib/ulan.h>
#include <keyval_id.h>
#include <uloi_base.h>
#include <lt_timer.h>
#include <uldy_base.h>
#if !defined(SDCC) 
  #include <keyval_loc.h>
  #include <lpciap_kvpb.h>
  #include <hal_machperiph.h>
#endif
#include "garage_gate.h"
#include "ul_idstr.h"

LT_TIMER_DEC(lt_100msec)
LT_TIMER_IMP(lt_100msec)

/***********************************/
// global variables
kvpb_block_t kvpb_block_global;
uloi_coninfo_t uloi_coninfo_global;
ul_dyac_t ul_dyac_global;
int tgate_off;

#ifndef UL_WITHOUT_HANDLE
kvpb_block_t *kvpb_block=&kvpb_block_global;
uloi_coninfo_t *coninfo=&uloi_coninfo_global;
ul_dyac_t *ul_dyac=&ul_dyac_global;
ul_fd_t ul_fd;
ul_fd_t ul_fd1;
ul_msginfo msginfo;
#endif /* UL_WITHOUT_HANDLE */

/* ulan variables */
unsigned char ubuff[30];
unsigned int  DATA uaddr;
unsigned long DATA usn;
uchar ustatus;

//object interface variables
unsigned int oi_gate_open;

/****************************************************************************/
int uloi_gate_open_wrfnc(ULOI_PARAM_coninfo void *context)
{
  uloi_uint_wrfnc(ULOI_ARG_coninfo context);
  if (oi_gate_open==1) {
    CLR_OUT_PIN(OUT_PORT,P1_26_GATE);
    tgate_off=5;
  }
  return 1;
}

/***********************************/
char ul_save_sn(uint32_t usn)
{
  kvpb_set_key(kvpb_block,KVPB_KEYID_ULAN_SN,sizeof(unsigned long),&usn);
  return 0;
}

/***********************************/
char ul_save_adr(uint8_t uaddr)
{
  unsigned int v=uaddr;
  kvpb_set_key(kvpb_block,KVPB_KEYID_ULAN_ADDR,sizeof(unsigned int),&v);
  return 0;
}

void sys_err()
{
  while(1);
}

int main(void) 
{

  /***********************************/
  // timer
  lt_100msec_init();

  /***********************************/
  //init values
  tgate_off=0;

  /***********************************/
  // kvpb init
  kvpb_block->base=(uint8_t CODE*)KEYVAL_START;
  kvpb_block->size=KEYVAL_PAGE_LEN;
  kvpb_block->flags=KVPB_DEFAULT_FLAGS;
 #ifndef  KVPB_MINIMALIZED
  kvpb_block->chunk_size=KVPB_CHUNK_SIZE;
  kvpb_block->erase=kvpb_erase;
  kvpb_block->copy=kvpb_copy;
  kvpb_block->flush=kvpb_flush;
 #endif
  if (kvpb_check(kvpb_block,1)<0) sys_err();

  // read data from kvpb
  uaddr=62;
  usn=0L;
  oi_gate_open=0;
  kvpb_get_key(kvpb_block,KVPB_KEYID_ULAN_ADDR,sizeof(unsigned int),&uaddr);
  kvpb_get_key(kvpb_block,KVPB_KEYID_ULAN_SN,sizeof(unsigned long),&usn);

  //********************
  // uLan init
 #ifdef UL_WITHOUT_HANDLE
  ul_drv_init();
  ul_drv_set_bdiv(BAUD2BAUDDIV(19200));
 #endif
  ul_fd=ul_open(NULL,NULL);
  if (ul_fd==UL_FD_INVALID) sys_err();    
 #ifndef UL_WITHOUT_HANDLE
  ul_fd1=ul_open(NULL,NULL);
  if (ul_fd1==UL_FD_INVALID) sys_err();    
 #endif
  ul_setidstr(ul_fd,ul_idstr);
  ul_setmyadr(ul_fd,uaddr);
  msginfo.sadr=0;
  msginfo.dadr=0;
  msginfo.cmd=0;
  ul_addfilt(ul_fd,&msginfo);
  ul_stroke(ul_fd);

  /***********************************/
  // uLan object interface init
  coninfo->cmd=UL_CMD_OISV;
  coninfo->sn=0;
  coninfo->bsn=0;
  coninfo->ul_fd=ul_fd;
 #ifndef UL_WITHOUT_HANDLE
  coninfo->ul_fd1=ul_fd1;
 #endif

  /***********************************/
  // uLan dyac init
  uldy_init(ul_dyac,ul_fd,ul_save_sn,ul_save_adr,(char*)ul_idstr,usn);

  //********************
  // start
 #ifdef SDCC 
  EA=1;       // Enable interrupts
 #endif

  while (1) {

    /* processing of ulan messages */
    if ((ul_inepoll(ul_fd)>0) && (ul_acceptmsg(ul_fd, &msginfo)>=0)) {
      if (!(msginfo.flg&(UL_BFL_PROC|UL_BFL_FAIL))) {
        if (uloi_process_msg(ULOI_ARG_coninfo (uloi_objdes_array_t*)&uloi_objdes_main, &msginfo)<0) {
          if (uldy_process_msg(ULDY_ARG_ul_dyac &msginfo)<0) {
            ul_freemsg(ul_fd);
          }
        }
      } else 
        ul_freemsg(ul_fd);
    }

    /* 100ms timer */
    if (lt_100msec_expired(100)) {
    
     if (tgate_off) {
       tgate_off--;    
       if (!tgate_off) {
         SET_OUT_PIN(OUT_PORT,P1_26_GATE);
       }
     }

      /* watchdog */
     #ifdef WATCHDOG_ENABLED
      watchdog_feed();
     #endif /* WATCHDOG_ENABLED */
    
      /* ulan stroke */
     #ifdef SDCC 
      ul_stroke(ul_fd);
     #endif
    }
  }
  return 0;
}
