#include <stdio.h>
#include <string.h>
#include <cpu_def.h> 
#include <system_def.h>
#include <lt_timer.h>
/* ulan */
#include <ul_lib/ulan.h>
/* USB support */
#include <usb/usbdebug.h>
#include <usb/lpc.h>
#include "ul_dcnv.h"
#include "u2u_vend.h"
#include <ul_idstr.h>

#define MASK_EP1RX  0x01
#define MASK_EP1TX  0x02

LT_TIMER_DEC(lt_10msec)
LT_TIMER_IMP(lt_10msec)

usb_device_t usb_device;
usb_ep_t eps[2]; 
ul_fd_t ul_fd_rx,ul_fd_tx;
ul_msginfo msginfo;
ul_dcnv_state_t ep1_dcnv_state;
uint8_t ulan_configured;
unsigned char ep1_rx_buff[USB_MAX_PACKET];
unsigned char ep1_tx_buff[USB_MAX_PACKET];
uint8_t timer_str,timer_rx_off,timer_tx_off,timer_configured; 

void timer_10ms(void) 
{
  if (timer_tx_off!=0) timer_tx_off--;
  else SET_OUT_PIN(LED_PORT,LED1_BIT);
  if (timer_rx_off!=0) timer_rx_off--;
  else SET_OUT_PIN(LED_PORT,LED2_BIT);
  if (!timer_str) {
    timer_str=5;
    ul_stroke(ul_fd_rx);
  } else timer_str--;
  if (timer_configured!=0) timer_configured--;
  else {
    timer_configured=20;
    if (!ulan_configured) {
      CLR_OUT_PIN(LED_PORT,LED1_BIT);
      CLR_OUT_PIN(LED_PORT,LED2_BIT);
      timer_rx_off=timer_tx_off=5;
    }
  }
}
    
int main() {  

  /***********************************/
  // timers 
  lt_10msec_init();

  //********************
  //uLan init
  ul_fd_rx=ul_open(NULL, NULL);
  ul_fd_tx=ul_open(NULL, NULL);
  ul_setidstr(ul_fd_rx,ul_idstr);
  ul_setmyadr(ul_fd_rx,2);
  msginfo.sadr=0;
  msginfo.dadr=0;
  msginfo.cmd=0;
  ul_addfilt(ul_fd_rx,&msginfo);
  ul_stroke(ul_fd_rx);
  //unconfigure ->
  ulan_configured=0;

  //********************
  // USB init
  memset( &usb_device, 0, sizeof( usb_device));
  usb_device.id = 1;
  usb_device.init = usb_lpc_init;
  usb_debug_set_level(DEBUG_LEVEL_NONE);
  usb_device.cntep = 2;
  usb_device.ep = eps;

  eps[0].max_packet_size = USB_MAX_PACKET;
  eps[1].max_packet_size = USB_MAX_PACKET;
  eps[0].epnum = 0x01;
  eps[1].epnum = 0x81;
  eps[0].event_mask = 0x04;
  eps[1].event_mask = 0x08;
  eps[0].udev = &usb_device;
  eps[1].udev = &usb_device;

  usb_device.vendor_fnc=usb_u2u_vendor;

  usb_init(&usb_device);
  usb_connect(&usb_device);

  //********************
  //start
  timer_rx_off=timer_tx_off=timer_str=timer_configured=0;
  ul_dcnv_init(&ep1_dcnv_state,ul_fd_tx,ul_fd_rx);

  while(1){
  
    usb_check_events(&usb_device);
    usb_control_response(&usb_device);
        
    if (usb_device.ep_events & MASK_EP1RX) {  //EP1RX
      int size;
      size=usb_udev_read_endpoint(&eps[0],ep1_rx_buff,USB_MAX_PACKET);
      ul_dcnv_send(&ep1_dcnv_state,ep1_rx_buff,size);
      usb_device.ep_events &= ~MASK_EP1RX;
      timer_tx_off=5; 		//rozsviceni diod pri vysilani 
      CLR_OUT_PIN(LED_PORT,LED1_BIT);
    }

    if(usb_device.ep_events & MASK_EP1TX){
      ep1_dcnv_state.rx_wait4host=0;
      usb_device.ep_events &= ~MASK_EP1TX;
    }

    if (!ep1_dcnv_state.rx_wait4host) { // EP1TX
      int size=ul_dcnv_rec(&ep1_dcnv_state, ep1_tx_buff, USB_MAX_PACKET);
      if (size){
        usb_udev_write_endpoint(&eps[1],ep1_tx_buff,size);
        ep1_dcnv_state.rx_wait4host=1;
      } else {
        if (ul_dcnv_rec_start(&ep1_dcnv_state, ep1_tx_buff,8)==8) { //HEADER
          usb_udev_write_endpoint(&eps[1],ep1_tx_buff,8);
          timer_rx_off=5;        //rosviceni diody pri prijmu 
          CLR_OUT_PIN(LED_PORT,LED2_BIT);
          ep1_dcnv_state.rx_wait4host=1;
        }
      }
    }
    
    /* 10ms timer */
    if (lt_10msec_expired(10)) 
      timer_10ms();
  }
}
